from enum import Enum


class TimeGranularity(str, Enum):
    SECOND = 'second'
    MINUTE = 'minute'
    HOUR = 'hour'
    DAY = 'day'
    WEEK = 'week'
    MONTH = 'month'
    YEAR = 'year'


class FilterOp(str, Enum):
    EQ = 'eq'
    LT = 'lt'
    GT = 'gt'
    LTE = 'lte'
    GTE = 'gte'
    IN = 'IN'
    LIKE = 'LIKE'
    NOT_NULL = 'not_null'
    IS_NULL = 'is_null'


class JoinType(str, Enum):
    INNER = 'inner'
    LEFT = 'left'
    RIGHT = 'right'
    OUTER = 'outer'


class MetricType(str, Enum):
    SQL = 'sql'
    SUM = 'sum'
    AVG = 'avg'
    MIN = 'min'
    MAX = 'max'
    COUNT = 'count'
    COUNT_DISTINCT = 'count_distinct'


class MetloSQLDataType(str, Enum):
    BOOLEAN = 'boolean'
    STRING = 'string'
    INT = 'int'
    FLOAT = 'float'
    DECIMAL = 'decimal'
    DATE = 'date'
    TIME_TZ = 'time_tz'
    TIME = 'time'
    TIMESTAMP_TZ = 'timestamp_tz'
    TIMESTAMP = 'timestamp'
    OTHER = 'other'



from datetime import datetime, date
import json


class DateTimeEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, (date, datetime)):
            return o.isoformat()
        return json.JSONEncoder.default(self, o)



import os
from typing import Optional

from dotenv import dotenv_values
from pydantic.dataclasses import dataclass


DEFAULT_CONFIG_FOLDER = os.path.join(os.path.expanduser('~/.metlo'))
DEFAULT_CONFIG_PATH = os.path.join(DEFAULT_CONFIG_FOLDER, 'credentials')
API_KEY_NAME = 'METLO_API_KEY'
HOST_KEY_NAME = 'METLO_HOST'
DEFINITION_DIR_KEY_NAME = 'METLO_DEFINITION_DIR'


@dataclass
class MetloConfig:
    api_key: str
    host_name: str
    definition_dir: Optional[str]


def load_config_file() -> Optional[dict]:
    config_path = os.environ.get('METLO_CONFIG_PATH', DEFAULT_CONFIG_PATH)
    if not os.path.exists(config_path):
        return None
    return dotenv_values(config_path)


def get_config() -> Optional[MetloConfig]:
    conf_file_vals = load_config_file() or {}
    api_key = os.environ.get(API_KEY_NAME, conf_file_vals.get(API_KEY_NAME))
    host_name = os.environ.get(HOST_KEY_NAME, conf_file_vals.get(HOST_KEY_NAME))
    definition_dir = os.environ.get(DEFINITION_DIR_KEY_NAME, conf_file_vals.get(DEFINITION_DIR_KEY_NAME))

    if not api_key:
        print('No API Key Specified')
    if not host_name:
        print('No Host Name Specified')
    
    if not (api_key and host_name):
        return
    
    return MetloConfig(
        api_key=api_key,
        host_name=host_name,
        definition_dir=definition_dir
    )



from glob import glob
import os
from typing import List

import yaml

from metlo.types.definition import Definition


def load_single_def(yaml_path: str) -> Definition:
    contents = open(yaml_path).read()
    yaml_data = yaml.load(contents, Loader=yaml.FullLoader)
    if 'id' not in yaml_data:
        yaml_data['id'] = yaml_path
    return Definition(**yaml_data)


def load_defs(yaml_dir: str) -> List[Definition]:
    yaml_paths = glob(os.path.join(yaml_dir, '*.yaml'))
    return [load_single_def(yaml_path) for yaml_path in yaml_paths]



from dataclasses import field
from typing import Optional

from pydantic.dataclasses import dataclass
from pydantic import ValidationError, validator

from metlo.types.enums import FilterOp, JoinType, MetloSQLDataType, MetricType


NO_VALUE_FILTER_OPS = {FilterOp.NOT_NULL, FilterOp.IS_NULL}


@dataclass
class Filter:
    column: str
    op: FilterOp
    value: Optional[str] = None

    @validator('value')
    def filter_op_requires_value(cls, v, values):
        filter_op = values.get('op')
        fails = filter_op not in NO_VALUE_FILTER_OPS and not v
        if fails:
            raise ValidationError(
                f'You need to specify a value for the {filter_op} filter.'
            )
        return v

    def __hash__(self):
        return hash(f'{self.column} {self.op} {self.value}')

    def __eq__(self, other):
        self_sql = f'{self.column} {self.op} {self.value}'
        other_sql = f'{other.column} {other.op} {other.value}'
        return self_sql == other_sql


@dataclass
class Dimension:
    sql: str
    description: str = ''
    primary_key: bool = False
    data_type: MetloSQLDataType = MetloSQLDataType.OTHER


@dataclass
class Metric:
    name: str
    description: str
    type: MetricType = MetricType.SQL
    sql: Optional[str] = ''
    filters: list[Filter] = field(default_factory=list)


@dataclass
class Join:
    table: str
    sql: str
    type: JoinType = JoinType.INNER


@dataclass
class Definition:
    id: str
    name: str
    owner: str
    description: str
    datasource: str
    table: str
    derived_table: bool = False
    joins: list[Join] = field(default_factory=list)
    metrics: list[Metric] = field(default_factory=list)
    dimensions: dict[str, Dimension] = field(default_factory=dict)
